"use strict";
Object.defineProperty(exports, "__esModule", {
    value: true
});
0 && (module.exports = {
    createDynamicallyTrackedSearchParams: null,
    createUntrackedSearchParams: null
});
function _export(target, all) {
    for(var name in all)Object.defineProperty(target, name, {
        enumerable: true,
        get: all[name]
    });
}
_export(exports, {
    createDynamicallyTrackedSearchParams: function() {
        return createDynamicallyTrackedSearchParams;
    },
    createUntrackedSearchParams: function() {
        return createUntrackedSearchParams;
    }
});
const _staticgenerationasyncstorageexternal = require("../../client/components/static-generation-async-storage.external");
const _dynamicrendering = require("../app-render/dynamic-rendering");
const _reflect = require("../web/spec-extension/adapters/reflect");
function createUntrackedSearchParams(searchParams) {
    const store = _staticgenerationasyncstorageexternal.staticGenerationAsyncStorage.getStore();
    if (store && store.forceStatic) {
        return {};
    } else {
        return searchParams;
    }
}
function createDynamicallyTrackedSearchParams(searchParams) {
    const store = _staticgenerationasyncstorageexternal.staticGenerationAsyncStorage.getStore();
    if (!store) {
        // we assume we are in a route handler or page render. just return the searchParams
        return searchParams;
    } else if (store.forceStatic) {
        // If we forced static we omit searchParams entirely. This is true both during SSR
        // and browser render because we need there to be parity between these environments
        return {};
    } else {
        // We need to track dynamic access with a Proxy. If `dynamic = "error"`, we use this information
        // to fail the build. This also signals to the patched fetch that it's inside
        // of a dynamic render and should bail from data cache. We implement get, has, and ownKeys because
        // these can all be used to exfiltrate information about searchParams.
        const trackedSearchParams = store.isStaticGeneration ? {} : searchParams;
        return new Proxy(trackedSearchParams, {
            get (target, prop, receiver) {
                if (typeof prop === 'string') {
                    (0, _dynamicrendering.trackDynamicDataAccessed)(store, `searchParams.${prop}`);
                }
                return _reflect.ReflectAdapter.get(target, prop, receiver);
            },
            has (target, prop) {
                if (typeof prop === 'string') {
                    (0, _dynamicrendering.trackDynamicDataAccessed)(store, `searchParams.${prop}`);
                }
                return Reflect.has(target, prop);
            },
            ownKeys (target) {
                (0, _dynamicrendering.trackDynamicDataAccessed)(store, 'searchParams');
                return Reflect.ownKeys(target);
            }
        });
    }
}

//# sourceMappingURL=search-params.js.map